==> vala-0.1.3.tar.bz2: vala-0.1.3/vala/valareturnstatement.c <==
/* valareturnstatement.vala
 *
 * Copyright (C) 2006-2007  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "valareturnstatement.h"
#include <vala/valaexpression.h>
#include <vala/valasourcereference.h>
#include <vala/valareturnstatement.h>
#include <vala/valacodevisitor.h>

struct _ValaReturnStatementPrivate {
	ValaExpression* _return_expression;
};
#define VALA_RETURN_STATEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_RETURN_STATEMENT, ValaReturnStatementPrivate))
enum  {
	VALA_RETURN_STATEMENT_DUMMY_PROPERTY,
	VALA_RETURN_STATEMENT_RETURN_EXPRESSION
};
static void vala_return_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_return_statement_real_replace (ValaCodeNode* base, ValaCodeNode* old_node, ValaCodeNode* new_node);
static gpointer vala_return_statement_parent_class = NULL;
static ValaStatementIface* vala_return_statement_vala_statement_parent_iface = NULL;
static void vala_return_statement_dispose (GObject * obj);


/**
 * Creates a new return statement.
 *
 * @param result the return expression
 * @param source reference to source code
 * @return       newly created return statement
 */
ValaReturnStatement* vala_return_statement_new (ValaExpression* result, ValaSourceReference* source)
{
	ValaReturnStatement * self;
	g_return_val_if_fail (result == NULL || VALA_IS_EXPRESSION (result), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	self = g_object_newv (VALA_TYPE_RETURN_STATEMENT, 0, NULL);
	vala_return_statement_set_return_expression (self, result);
	vala_code_node_set_source_reference (VALA_CODE_NODE (self), source);
	return self;
}


static void vala_return_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaReturnStatement * self;
	self = ((ValaReturnStatement*) base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_begin_return_statement (visitor, self);
	if (vala_return_statement_get_return_expression (self) != NULL) {
		vala_code_node_accept (VALA_CODE_NODE (vala_return_statement_get_return_expression (self)), visitor);
		vala_code_visitor_visit_end_full_expression (visitor, vala_return_statement_get_return_expression (self));
	}
	vala_code_visitor_visit_end_return_statement (visitor, self);
}


static void vala_return_statement_real_replace (ValaCodeNode* base, ValaCodeNode* old_node, ValaCodeNode* new_node)
{
	ValaReturnStatement * self;
	self = ((ValaReturnStatement*) base);
	g_return_if_fail (VALA_IS_CODE_NODE (old_node));
	g_return_if_fail (VALA_IS_CODE_NODE (new_node));
	if (VALA_CODE_NODE (vala_return_statement_get_return_expression (self)) == old_node) {
		vala_return_statement_set_return_expression (self, VALA_EXPRESSION (new_node));
	}
}


ValaExpression* vala_return_statement_get_return_expression (ValaReturnStatement* self)
{
	g_return_val_if_fail (VALA_IS_RETURN_STATEMENT (self), NULL);
	return self->priv->_return_expression;
}


void vala_return_statement_set_return_expression (ValaReturnStatement* self, ValaExpression* value)
{
	ValaExpression* _tmp2;
	ValaExpression* _tmp1;
	g_return_if_fail (VALA_IS_RETURN_STATEMENT (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_return_expression = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_return_expression == NULL ? NULL : (self->priv->_return_expression = (g_object_unref (self->priv->_return_expression), NULL))), _tmp2);
	if (self->priv->_return_expression != NULL) {
		vala_code_node_set_parent_node (VALA_CODE_NODE (self->priv->_return_expression), VALA_CODE_NODE (self));
	}
}


static void vala_return_statement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaReturnStatement * self;
	self = VALA_RETURN_STATEMENT (object);
	switch (property_id) {
		case VALA_RETURN_STATEMENT_RETURN_EXPRESSION:
		g_value_set_object (value, vala_return_statement_get_return_expression (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_return_statement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaReturnStatement * self;
	self = VALA_RETURN_STATEMENT (object);
	switch (property_id) {
		case VALA_RETURN_STATEMENT_RETURN_EXPRESSION:
		vala_return_statement_set_return_expression (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_return_statement_class_init (ValaReturnStatementClass * klass)
{
	vala_return_statement_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaReturnStatementPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_return_statement_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_return_statement_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_return_statement_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_return_statement_real_accept;
	VALA_CODE_NODE_CLASS (klass)->replace = vala_return_statement_real_replace;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_RETURN_STATEMENT_RETURN_EXPRESSION, g_param_spec_object ("return-expression", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_return_statement_vala_statement_interface_init (ValaStatementIface * iface)
{
	vala_return_statement_vala_statement_parent_iface = g_type_interface_peek_parent (iface);
}


static void vala_return_statement_init (ValaReturnStatement * self)
{
	self->priv = VALA_RETURN_STATEMENT_GET_PRIVATE (self);
}


static void vala_return_statement_dispose (GObject * obj)
{
	ValaReturnStatement * self;
	ValaReturnStatementClass * klass;
	GObjectClass * parent_class;
	self = VALA_RETURN_STATEMENT (obj);
	(self->priv->_return_expression == NULL ? NULL : (self->priv->_return_expression = (g_object_unref (self->priv->_return_expression), NULL)));
	klass = VALA_RETURN_STATEMENT_CLASS (g_type_class_peek (VALA_TYPE_RETURN_STATEMENT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_return_statement_get_type (void)
{
	static GType vala_return_statement_type_id = 0;
	if (G_UNLIKELY (vala_return_statement_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaReturnStatementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_return_statement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaReturnStatement), 0, (GInstanceInitFunc) vala_return_statement_init };
		static const GInterfaceInfo vala_statement_info = { (GInterfaceInitFunc) vala_return_statement_vala_statement_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		vala_return_statement_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaReturnStatement", &g_define_type_info, 0);
		g_type_add_interface_static (vala_return_statement_type_id, VALA_TYPE_STATEMENT, &vala_statement_info);
	}
	return vala_return_statement_type_id;
}

==> vala-0.3.1.tar.bz2: vala-0.3.1/vala/valareturnstatement.c <==
/* valareturnstatement.vala
 *
 * Copyright (C) 2006-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <vala/valareturnstatement.h>
#include <vala/valaexpression.h>
#include <vala/valasourcereference.h>
#include <vala/valacodevisitor.h>




struct _ValaReturnStatementPrivate {
	ValaExpression* _return_expression;
};
#define VALA_RETURN_STATEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_RETURN_STATEMENT, ValaReturnStatementPrivate))
enum  {
	VALA_RETURN_STATEMENT_DUMMY_PROPERTY,
	VALA_RETURN_STATEMENT_RETURN_EXPRESSION
};
static void vala_return_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_return_statement_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_return_statement_real_replace_expression (ValaCodeNode* base, ValaExpression* old_node, ValaExpression* new_node);
static gpointer vala_return_statement_parent_class = NULL;
static ValaStatementIface* vala_return_statement_vala_statement_parent_iface = NULL;
static void vala_return_statement_dispose (GObject * obj);



/**
 * Creates a new return statement.
 *
 * @param return_expression the return expression
 * @param source_reference  reference to source code
 * @return                  newly created return statement
 */
ValaReturnStatement* vala_return_statement_new (ValaExpression* return_expression, ValaSourceReference* source_reference) {
	ValaReturnStatement * self;
	g_return_val_if_fail (return_expression == NULL || VALA_IS_EXPRESSION (return_expression), NULL);
	g_return_val_if_fail (source_reference == NULL || VALA_IS_SOURCE_REFERENCE (source_reference), NULL);
	self = g_object_newv (VALA_TYPE_RETURN_STATEMENT, 0, NULL);
	vala_code_node_set_source_reference (VALA_CODE_NODE (self), source_reference);
	vala_return_statement_set_return_expression (self, return_expression);
	return self;
}


static void vala_return_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
	ValaReturnStatement * self;
	self = VALA_RETURN_STATEMENT (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_return_statement (visitor, self);
}


static void vala_return_statement_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
	ValaReturnStatement * self;
	self = VALA_RETURN_STATEMENT (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	if (vala_return_statement_get_return_expression (self) != NULL) {
		vala_code_node_accept (VALA_CODE_NODE (vala_return_statement_get_return_expression (self)), visitor);
		vala_code_visitor_visit_end_full_expression (visitor, vala_return_statement_get_return_expression (self));
	}
}


static void vala_return_statement_real_replace_expression (ValaCodeNode* base, ValaExpression* old_node, ValaExpression* new_node) {
	ValaReturnStatement * self;
	self = VALA_RETURN_STATEMENT (base);
	g_return_if_fail (VALA_IS_EXPRESSION (old_node));
	g_return_if_fail (VALA_IS_EXPRESSION (new_node));
	if (vala_return_statement_get_return_expression (self) == old_node) {
		vala_return_statement_set_return_expression (self, new_node);
	}
}


ValaExpression* vala_return_statement_get_return_expression (ValaReturnStatement* self) {
	g_return_val_if_fail (VALA_IS_RETURN_STATEMENT (self), NULL);
	return self->priv->_return_expression;
}


void vala_return_statement_set_return_expression (ValaReturnStatement* self, ValaExpression* value) {
	ValaExpression* _tmp2;
	ValaExpression* _tmp1;
	g_return_if_fail (VALA_IS_RETURN_STATEMENT (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_return_expression = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_return_expression == NULL ? NULL : (self->priv->_return_expression = (g_object_unref (self->priv->_return_expression), NULL))), _tmp2);
	if (self->priv->_return_expression != NULL) {
		vala_code_node_set_parent_node (VALA_CODE_NODE (self->priv->_return_expression), VALA_CODE_NODE (self));
	}
}


static void vala_return_statement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValaReturnStatement * self;
	self = VALA_RETURN_STATEMENT (object);
	switch (property_id) {
		case VALA_RETURN_STATEMENT_RETURN_EXPRESSION:
		g_value_set_object (value, vala_return_statement_get_return_expression (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_return_statement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValaReturnStatement * self;
	self = VALA_RETURN_STATEMENT (object);
	switch (property_id) {
		case VALA_RETURN_STATEMENT_RETURN_EXPRESSION:
		vala_return_statement_set_return_expression (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_return_statement_class_init (ValaReturnStatementClass * klass) {
	vala_return_statement_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaReturnStatementPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_return_statement_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_return_statement_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_return_statement_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_return_statement_real_accept;
	VALA_CODE_NODE_CLASS (klass)->accept_children = vala_return_statement_real_accept_children;
	VALA_CODE_NODE_CLASS (klass)->replace_expression = vala_return_statement_real_replace_expression;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_RETURN_STATEMENT_RETURN_EXPRESSION, g_param_spec_object ("return-expression", "return-expression", "return-expression", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_return_statement_vala_statement_interface_init (ValaStatementIface * iface) {
	vala_return_statement_vala_statement_parent_iface = g_type_interface_peek_parent (iface);
}


static void vala_return_statement_init (ValaReturnStatement * self) {
	self->priv = VALA_RETURN_STATEMENT_GET_PRIVATE (self);
}


static void vala_return_statement_dispose (GObject * obj) {
	ValaReturnStatement * self;
	self = VALA_RETURN_STATEMENT (obj);
	(self->priv->_return_expression == NULL ? NULL : (self->priv->_return_expression = (g_object_unref (self->priv->_return_expression), NULL)));
	G_OBJECT_CLASS (vala_return_statement_parent_class)->dispose (obj);
}


GType vala_return_statement_get_type (void) {
	static GType vala_return_statement_type_id = 0;
	if (G_UNLIKELY (vala_return_statement_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaReturnStatementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_return_statement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaReturnStatement), 0, (GInstanceInitFunc) vala_return_statement_init };
		static const GInterfaceInfo vala_statement_info = { (GInterfaceInitFunc) vala_return_statement_vala_statement_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		vala_return_statement_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaReturnStatement", &g_define_type_info, 0);
		g_type_add_interface_static (vala_return_statement_type_id, VALA_TYPE_STATEMENT, &vala_statement_info);
	}
	return vala_return_statement_type_id;
}
